<?php

/*
 * This file is part of the Kreta package.
 *
 * (c) Beñat Espiña <benatespina@gmail.com>
 * (c) Gorka Laucirica <gorka.lauzirika@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace spec\Kreta\Bundle\OrganizationBundle\Security\Authorization\Voter;

use Kreta\Component\Issue\Model\Interfaces\IssueInterface;
use Kreta\Component\Organization\Model\Interfaces\OrganizationInterface;
use Kreta\Component\Organization\Model\Interfaces\ParticipantInterface;
use Kreta\Component\Organization\Model\Organization;
use Kreta\Component\User\Model\Interfaces\UserInterface;
use PhpSpec\ObjectBehavior;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;

/**
 * Class OrganizationVoterSpec.
 *
 * @author Beñat Espiña <benatespina@gmail.com>
 * @author Gorka Laucirica <gorka.lauzirika@gmail.com>
 */
class OrganizationVoterSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType('Kreta\Bundle\OrganizationBundle\Security\Authorization\Voter\OrganizationVoter');
    }

    function it_extends_abstract_voter()
    {
        $this->shouldHaveType('Kreta\Bundle\CoreBundle\Security\Authorization\Voter\Abstracts\AbstractVoter');
    }

    function it_implements_voter_interface()
    {
        $this->shouldImplement('Symfony\Component\Security\Core\Authorization\Voter\VoterInterface');
    }

    function it_supports_attribute()
    {
        $this->supportsAttribute('add_participant')->shouldReturn(true);
    }

    function it_does_not_support_attribute()
    {
        $this->supportsAttribute('non-exist-attribute')->shouldReturn(false);
    }

    function it_supports_class()
    {
        $this->supportsClass('Kreta\Component\Organization\Model\Interfaces\OrganizationInterface')->shouldReturn(true);
    }

    function it_does_not_support_class()
    {
        $this->supportsClass('Kreta\Component\Project\Model\Interfaces\ProjectInterface')->shouldReturn(false);
    }

    function it_does_not_vote_because_it_does_not_support_class(TokenInterface $token, IssueInterface $issue)
    {
        $this->vote($token, $issue, [])->shouldReturn(0);
    }

    function it_does_not_vote_because_it_only_one_attribute_allowed(TokenInterface $token, Organization $organization)
    {
        $this->shouldThrow(new \InvalidArgumentException('Only one attribute allowed.'))
            ->during('vote', [$token, $organization, []]);
    }

    function it_does_not_vote_because_it_does_not_support_attribute(TokenInterface $token, Organization $organization)
    {
        $this->vote($token, $organization, ['non-exist-attribute'])->shouldReturn(0);
    }

    function it_does_not_vote_because_the_current_user_is_not_user_interface_instance(
        TokenInterface $token,
        Organization $organization
    ) {
        $token->getUser()->shouldBeCalled()->willReturn(null);

        $this->vote($token, $organization, ['add_participant'])->shouldReturn(-1);
    }

    function it_does_not_vote_add_participant_delete_delete_participant_or_edit_grant(
        TokenInterface $token,
        Organization $organization,
        UserInterface $user
    ) {
        $token->getUser()->shouldBeCalled()->willReturn($user);
        $organization->getUserRole($user)->shouldBeCalled()->willReturn(ParticipantInterface::ORG_PARTICIPANT);

        $this->vote($token, $organization, ['add_participant'])->shouldReturn(-1);
    }

    function it_does_not_vote_view_grant(TokenInterface $token, Organization $organization, UserInterface $user)
    {
        $token->getUser()->shouldBeCalled()->willReturn($user);
        $organization->getUserRole($user)->shouldBeCalled()->willReturn(null);

        $this->vote($token, $organization, ['view'])->shouldReturn(-1);
    }

    function it_votes_add_participant_delete_delete_participant_or_edit_because_its_role_organization_is_admin(
        TokenInterface $token,
        UserInterface $user,
        Organization $organization
    ) {
        $token->getUser()->shouldBeCalled()->willReturn($user);
        $organization->getUserRole($user)->shouldBeCalled()->willReturn(ParticipantInterface::ORG_ADMIN);

        $this->vote($token, $organization, ['add_participant'])->shouldReturn(1);
    }

    function it_votes(TokenInterface $token, UserInterface $user, OrganizationInterface $organization)
    {
        $token->getUser()->shouldBeCalled()->willReturn($user);
        $organization->getUserRole($user)->shouldBeCalled()->willReturn(ParticipantInterface::ORG_PARTICIPANT);

        $this->vote($token, $organization, ['view'])->shouldReturn(1);
    }
}
