<?php

/*
 * This file is part of the Kreta package.
 *
 * (c) Beñat Espiña <benatespina@gmail.com>
 * (c) Gorka Laucirica <gorka.lauzirika@gmail.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Kreta\Component\Organization\Model;

use Doctrine\Common\Collections\ArrayCollection;
use EasySlugger\Slugger;
use Kreta\Component\Media\Model\Interfaces\MediaInterface;
use Kreta\Component\Organization\Model\Interfaces\OrganizationInterface;
use Kreta\Component\Organization\Model\Interfaces\ParticipantInterface;
use Kreta\Component\Project\Model\Interfaces\ProjectInterface;
use Kreta\Component\User\Model\Interfaces\UserInterface;

/**
 * Organization model class.
 *
 * @author Beñat Espiña <benatespina@gmail.com>
 */
class Organization implements OrganizationInterface
{
    /**
     * The id.
     *
     * @var string
     */
    protected $id;

    /**
     * The image.
     *
     * @var MediaInterface
     */
    protected $image;

    /**
     * The name.
     *
     * @var string
     */
    protected $name;

    /**
     * Array that contains all the
     * participants of the organization.
     *
     * @var ParticipantInterface[]
     */
    protected $participants;

    /**
     * Array that contains all the
     * projects of the organization.
     *
     * @var ProjectInterface[]
     */
    protected $projects;

    /**
     * The slug.
     *
     * @var string
     */
    protected $slug;

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->participants = new ArrayCollection();
        $this->projects = new ArrayCollection();
    }

    /**
     * {@inheritdoc}
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * {@inheritdoc}
     */
    public function getImage()
    {
        return $this->image;
    }

    /**
     * {@inheritdoc}
     */
    public function setImage(MediaInterface $image)
    {
        $this->image = $image;

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * {@inheritdoc}
     */
    public function setName($name)
    {
        $this->name = $name;
        $this->setSlug(Slugger::slugify($name));

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getParticipants()
    {
        return $this->participants;
    }

    /**
     * {@inheritdoc}
     */
    public function addParticipant(ParticipantInterface $participant)
    {
        $projects = $participant->getOrganization()->getProjects();
        $user = $participant->getUser();
        foreach ($projects as $key => $project) {
            foreach ($project->getParticipants() as $projectParticipant) {
                if ($projectParticipant->getUser()->getId() === $user->getId()) {
                    $participant->getOrganization()->getProjects()[$key]->removeParticipant($projectParticipant);
                }
            }
        }

        $this->participants[] = $participant;

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function removeParticipant(ParticipantInterface $participant)
    {
        $this->participants->removeElement($participant);

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getProjects()
    {
        return $this->projects;
    }

    /**
     * {@inheritdoc}
     */
    public function addProject(ProjectInterface $project)
    {
        $this->projects[] = $project;

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function removeProject(ProjectInterface $project)
    {
        $this->projects->removeElement($project);

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getSlug()
    {
        return $this->slug;
    }

    /**
     * {@inheritdoc}
     */
    public function setSlug($slug)
    {
        $this->slug = $slug;

        return $this;
    }

    /**
     * {@inheritdoc}
     */
    public function getUserRole(UserInterface $user)
    {
        foreach ($this->participants as $participant) {
            if ($participant->getUser()->getId() === $user->getId()) {
                return $participant->getRole();
            }
        }

        return;
    }

    /**
     * Magic method that is useful in Twig templates
     * representing the entity class into string.
     *
     * @return string
     */
    public function __toString()
    {
        return $this->name;
    }
}
